import { Center, Box, VStack, Button, HStack, Text, Image, ScrollView, View, Spacer, Card, Heading, Switch, Flex, ChevronLeftIcon, ChevronRightIcon, Divider, Pressable } from "native-base"
import React, { useCallback, useEffect, useState } from "react"
import { Item } from "../../types/item"
import { Note } from "../../types/note"
import LoadingBox from "../loading/LoadingBox"
import NotesListView from "../notes/NotesListView"
import { IMAGE_URL } from "../../api/constants"
import { log } from "../../logging/logger"
import { createNote, getAllNotes } from "../../stores/actions/notesThunks"
import { AppDispatch, RootState } from "../../stores/store"
import { useDispatch, useSelector } from "react-redux"
import { getItemNotes } from "../../stores/actions/itemThunks"

import { Dimensions } from "react-native";
import { DrawerNavigationProp } from "@react-navigation/drawer"
import { RootStackParamList } from "../../pages/Navigation"
import { select } from "d3"

interface ItemDetailProps {
    item: Item,
    itemLoading: boolean,
    notes: Note[],
    notesLoading: boolean,
    navigation: DrawerNavigationProp<RootStackParamList, "Item", undefined>
}

const ItemDetail = (props: ItemDetailProps) => {
    const item = props.item;

    const [selectedImage, setSelectedImage] = useState<number>(0);

    const [images, setImages] = useState<{ imageUrl: string, title: string }[]>([]);


    useEffect(() => {
        if (item.images?.[0]) {
            setImages([item.images[0], item.images[0], item.images[0], item.images[0], item.images[0], item.images[0], item.images[0]])
        }
    }, [item])

    return (<>
        {
            item && props.itemLoading ?
                (<LoadingBox text="Item loading..." />)
                :
                (
                    <ScrollView flex="1" paddingLeft={2.5} paddingRight={2.5}>
                        <VStack>
                            <HStack marginLeft={2.5} alignItems={"center"}>
                                <VStack flex={1} marginTop={3.5}>
                                    <Text fontSize={13} color="#4D4D4D">
                                        {item.authorDisplayName ? item.authorDisplayName : item.authorName}
                                    </Text>
                                    <Heading marginTop={1} fontSize={18} color="#4D4D4D">
                                        {item.workName}
                                    </Heading>

                                </VStack>
                                <Box marginLeft="auto" marginTop={0}> {/* marginLeft: "auto" pushes the Box to the right */}
                                    <Button variant="outline" backgroundColor={"#F4DFAB"} borderRadius={8} size="sm" padding={1.5} >
                                        <Text color="#654B07" fontSize={14}>
                                            {item?.concordances?.[0]?.id}
                                        </Text>
                                    </Button>
                                </Box>
                            </HStack>


                            <View marginTop={5} backgroundColor="#F5F4F1" borderRadius={10} borderColor="#F5F4F1" padding={2}>
                                {item.inventoryItem &&
                                    <VStack>
                                        <Text italic>
                                            <Text color="#654B07" italic fontWeight={"semibold"}>Inventory item: </Text>
                                            {item.inventoryItem}
                                        </Text>
                                        <Text italic marginTop="2.5%">
                                            {item.searchSubjects != undefined && item?.searchSubjects?.map((subject, index) => (
                                                subject != "" ?
                                                    item.searchSubjects && index < item.searchSubjects.length - 1 ? (subject + ", ") : (subject)
                                                    :
                                                    ""
                                            ))}
                                        </Text>
                                    </VStack>}
                            </View>
                            {item.fullView && item.images && item.images.length > 0 && (
                                <VStack marginTop={5} alignItems="center" justifyContent="center" backgroundColor="#F5F4F1" borderRadius={10} borderColor="#F5F4F1" padding={2}>

                                    <Box marginTop={2.5} borderWidth={1} width={Dimensions.get('window').width - 110} alignItems="center" backgroundColor="red" borderTopRadius={10} >
                                        <Text marginBottom={2}>{"Resembling objects"}
                                            {selectedImage + 1 + "/" + item.images.length + item.images[selectedImage].cert + item.images[selectedImage].relationship_type}
                                        </Text>
                                    </Box>

                                    <HStack space={4}>
                                        <Button variant="ghost" size="lg" onPress={() => {
                                            if (item.images && selectedImage - 1 >= 0) {
                                                setSelectedImage(selectedImage - 1);
                                            }
                                        }} leftIcon={<ChevronLeftIcon size="lg" />} />
                                        <Image size={Dimensions.get('window').width - 110} alt="image" source={{ uri: IMAGE_URL + "/" + item.images[selectedImage].imageUrl }} />
                                        <Button variant="ghost" size="lg" onPress={() => {
                                            if (item.images && selectedImage + 1 < item.images.length) {
                                                setSelectedImage(selectedImage + 1);
                                            }
                                        }} leftIcon={<ChevronRightIcon size="lg" />} />
                                    </HStack>
                                    <ScrollView horizontal marginBottom={2.5} marginTop={5}>

                                        {item.images.map((image, index) => (
                                            <Pressable
                                                onPress={() => { setSelectedImage(index); }}>
                                                <Image marginLeft={2.5} size={90} alt="image" source={{ uri: IMAGE_URL + "/" + image.imageUrl }}
                                                    borderColor={selectedImage == index ? "#1782FF" : undefined}
                                                    borderWidth={3}
                                                />
                                            </Pressable>

                                        ))}
                                    </ScrollView>
                                </VStack>
                            )}
                            {item.fullView && (
                                <View marginTop={5} backgroundColor="#F5F4F1" borderRadius={10} borderColor="#F5F4F1" padding={2} marginBottom={5}>
                                    {item.authorName &&
                                        <HStack>
                                            <Text marginTop={2.5}>
                                                <Text color="#654B07" italic fontWeight={"semibold"}>Author name: </Text>
                                                {item.authorName}
                                            </Text>
                                        </HStack>}
                                    {item.title &&
                                        <HStack>
                                            <Text marginTop={2.5}>
                                                <Text color="#654B07" italic fontWeight={"semibold"}>Title: </Text>
                                                {item.title}
                                            </Text>
                                        </HStack>}
                                    {item.institution &&
                                        <HStack>
                                            <Text marginTop={2.5}>
                                                <Text color="#654B07" italic fontWeight={"semibold"}>Institution: </Text>
                                                {item.institution.name}, Inv. No. {item.institution.inventoryNumber}, {item.institution.city} {"(" + item.institution.country + ")"}
                                            </Text>
                                        </HStack>}
                                    {item.repository &&
                                        <HStack>
                                            <Text marginTop={2.5}>
                                                <Text color="#654B07" italic fontWeight={"semibold"}>Repository: </Text>
                                                {item.repository}
                                            </Text>
                                        </HStack>}
                                    {item.provenance &&
                                        <HStack>
                                            <Text marginTop={2.5}>
                                                <Text color="#654B07" italic fontWeight={"semibold"}>Provenance: </Text>
                                                {item.provenance}
                                            </Text>
                                        </HStack>}
                                    {item.description &&
                                        <HStack>
                                            <Text marginTop={2.5}>
                                                <Text color="#654B07" italic fontWeight={"semibold"}>Description: </Text>
                                                {item.description}
                                            </Text>
                                        </HStack>}
                                </View>)}

                            {
                                item.room && item.room.id &&
                                <Flex direction="row" alignItems="center" justify="flex-end">
                                    <Button
                                        onPress={() => props.navigation.navigate("Plan", { roomId: item?.room?.id, placeId: undefined })}
                                        variant={"link"}
                                        backgroundColor={"#F4DFAB"}
                                        borderRadius={7}
                                        marginTop={0}
                                        marginBottom={5}
                                        width={125}
                                        rightIcon={<ChevronRightIcon size="xs" />}>
                                        Show in map
                                    </Button>
                                </Flex>

                            }

                        </VStack>
                    </ScrollView >
                )
        }</>
    );
}



const ItemNotes = (props: { item: Item, notes: Note[], notesLoading: boolean, navigation: DrawerNavigationProp<RootStackParamList, "Item", undefined> }) => {

    const [showRelatedComments, setShowRelatedComments] = useState(true);

    const { requestPending, triggerRefresh } = useSelector((state: RootState) => state.noteViewState)

    const dispatch = useDispatch<AppDispatch>();

    // trigger refresh on triggerRefresh increment
    useEffect(() => {
        if (triggerRefresh > 0) {
            log.debug("NotesViewPage", "useEffect", "getNotes");
            dispatch(getItemNotes({ item: props.item, relatedComments: showRelatedComments }));
        }
    }, [triggerRefresh])

    const toggleRelatedComments = useCallback(() => {
        if (!props.notesLoading) {

            let value = !showRelatedComments;

            setShowRelatedComments(value);
            dispatch(getItemNotes({ item: props.item, relatedComments: value }));
        }
    }, [showRelatedComments]);


    const handleCreateItemComment = useCallback(
        (newComment: string, replyingTo: { messageId: string; userName: string } | null) => {
            if (!requestPending) {
                // creating new comment
                if (replyingTo != null) {
                    dispatch(createNote({ newNote: { note: newComment, reply_to: replyingTo.messageId } as Note }));
                } else {
                    dispatch(createNote({ newNote: { note: newComment, items: props.item.id } as any }));
                }
            }
        },
        [props.item.id]
    );

    return (

        props.notesLoading ?
            (<LoadingBox text="Notes loading..." />)
            :
            (<VStack paddingLeft={2.5} paddingRight={2.5}>
                <Flex direction="row" alignItems="center" justify="flex-end">
                    <Text fontSize={"12"}>Show related comments</Text>
                    <Switch value={showRelatedComments} onChange={toggleRelatedComments} size="md" />
                </Flex>
                <Box >
                    <NotesListView height={Dimensions.get('window').height - 355} requestPending={requestPending} notes={props.notes} navigation={props.navigation} handleCreateComment={handleCreateItemComment} />
                </Box>
            </VStack>)
    );
}



const ItemView = (props: ItemDetailProps) => {

    // item shown / note shown
    const [itemShown, setItemShown] = React.useState(true);

    const { itemLoading, notesLoading, navigation } = props;

    const handleItemClick = () => {
        setItemShown(true);
    }

    const handleNotesClick = () => {
        setItemShown(false);
    }

    return (
        <Box>

            <Box height={Dimensions.get('window').height - 210}>
                {itemShown ? (
                    <ItemDetail itemLoading={props.itemLoading} item={props.item} notesLoading={props.notesLoading} notes={props.notes} navigation={props.navigation} />
                ) : (
                    <ItemNotes item={props.item} notesLoading={props.notesLoading} notes={props.notes} navigation={props.navigation} />
                )}
            </Box>

            {/* item notes switch tab */}
            <HStack alignItems="center">
                <Button
                    variant={"unstyled"}
                    w="50%"
                    onPress={handleItemClick}
                >
                    <VStack alignItems={"center"}>
                        <Text color="primary" bold={itemShown} mb={itemShown ? undefined : 2}>
                            Item
                        </Text>
                        {itemShown && <Divider borderWidth={1} width={60} mt={2} color="primary" />}
                    </VStack>
                </Button>
                <Button
                    variant={"unstyled"}
                    w="50%"
                    onPress={handleNotesClick}
                >
                    <VStack alignItems={"center"}>
                        <Text color="primary" bold={!itemShown} mb={!itemShown ? undefined : 2}>
                            Notes
                        </Text>
                        {!itemShown && <Divider borderWidth={1} width={60} mt={2} color="primary" />}
                    </VStack>
                </Button>
            </HStack>
        </Box>
    )
}

export default ItemView